/*
  Sharp MZ-3500 Keyboard Emulator
  
  (c) Martin Lukasek, www.8bity.cz
  
 */
 
#include "PS2Keyboard.h"

const int DataPin = 8;  // ATMega328 pin 14
const int IRQpin =  2;  // ATMega328 pin 4

PS2Keyboard keyboard;
 
int kbdRetransmissions = 1000;
int nrInts = 0;

// Pin 13 has an LED connected on most Arduino boards.
// give it a name:
int led = 13;    // ATMega328 pin 28
int pinDK = 5;   // ATMega328 pin 11
int pinSTK = 6;  // ATMega328 pin 12
int pinACKC = 4; // ATMega328 pin 6
int pinDC = 7;   // ATMega328 pin 13
int pinSTC = 3;  // ATMega328 pin 5

// the setup routine runs once when you press reset:
void setup() {                
  // initialize the digital pin as an output.
  pinMode(led, OUTPUT);     
  pinMode(pinDK, OUTPUT);     
  digitalWrite(pinDK, LOW);		// HIGH ?
  pinMode(pinSTK, OUTPUT);     
  digitalWrite(pinSTK, LOW);		
  pinMode(pinACKC, INPUT);     
  digitalWrite(pinACKC, LOW);		
  pinMode(pinDC, INPUT);     
  digitalWrite(pinDC, LOW);		
  pinMode(pinSTC, INPUT);     
  digitalWrite(pinSTC, LOW);		
  
  delay(10);
  keyboard.begin(DataPin, IRQpin, PS2Keymap_MZ3500);

  Serial.begin(115200); 
  Serial.println("Sharp MZ-3500 Arduino Keyboard Adapter   (c) 2013 Martin Lukasek, www.8bity.cz");

  attachInterrupt(1, intSTC, FALLING);

  keyboard.reset(); // Initialize PS/2 keayboard, set Num Lock ON
  kbd_lck_state |= LED_NUMLOCK;
  kbd_set_lights(kbd_lck_state);

  sendKey(0xF2,1); // keyboard PRO mode
}

void intSTC() {
  // Serial.println("INT STC");
  nrInts++;
}

void strobePulse() { // 10 + 17,5 pulse + 45
  delayMicroseconds(5); // exp 3, meas 6
  digitalWrite(pinSTK, HIGH);
  delayMicroseconds(12); // exp 10, meas 13
  digitalWrite(pinSTK, LOW);		
  delayMicroseconds(33); // exp 10, meas 39
}

void strobeIntPulse() { // 0 + 10 pulse + 25
  digitalWrite(pinSTK, HIGH);
  delayMicroseconds(6); // exp 5, meas 7
  digitalWrite(pinSTK, LOW);		
  delayMicroseconds(17); // exp 12, meas 19
}
void sendByte(uint8_t dt, uint8_t cmd) {
  uint8_t i, dtBit, msk;
  uint8_t parity = 0;
  
  // sync scope - just for testing
  // digitalWrite(pinSTK, HIGH);
  // digitalWrite(pinSTK, LOW);		

  // dt = ~dt; // negative logic on the cable
  msk = 0x80;
  
  noInterrupts();
  
  digitalWrite(pinSTK, LOW);		
  digitalWrite(pinDK, LOW);		
  strobeIntPulse();
  
  for (i = 0; i < 8; i++)  {
    dtBit = !!(dt & msk);
    msk = msk >> 1;
    digitalWrite(pinDK, dtBit);
    strobePulse();
    parity ^= dtBit;
  }
  dtBit = !!(cmd!=0);
  digitalWrite(pinDK, dtBit);
  strobePulse();
  parity ^= dtBit;

  digitalWrite(pinDK, parity);
  strobePulse();
  
  digitalWrite(pinDK, LOW);
  digitalWrite(pinSTK, LOW);	

  interrupts();	
}

void sendKey(uint8_t dt, uint8_t cmd) {
  int rtry = 0;
  uint8_t ACKC;
  unsigned long tm;
  int tmRetry = 1250;
  while( rtry<kbdRetransmissions ) {
    sendByte(dt, cmd);
    tm = micros();
    ACKC = digitalRead(pinACKC);
    // digitalWrite(pinDK, LOW);
    // digitalWrite(pinDK, HIGH);	
    while( (ACKC==0) && (micros()-tm<tmRetry) ) {
      delayMicroseconds(5);
      ACKC = digitalRead(pinACKC);
    };
    if( ACKC==1 ) {
/*      Serial.print(micros()-tm);
      Serial.print(" - ");
      Serial.println(rtry); */
      return;
    }
    if( tmRetry == 1260 )
      tmRetry = 3420;
    else
      tmRetry = 1260;
    rtry++;
  }
  Serial.print("KBD ERR (>1000 retransmissions): ");
  Serial.print(micros()-tm);
  Serial.print("us, ");
  Serial.print(rtry);
  Serial.println(" attemps");
}

// the loop routine runs over and over again forever:
void loop() {
  char c;
  byte ccmd;
  
  while( true )  
  {

    if (keyboard.available()) {
      // read the next key
      c = keyboard.read();
      ccmd = 0;

      Serial.print(">");
      Serial.print(byte(c));
      Serial.print("<");
      
// check for some of the special keys

      if ((byte(c)>=0xA0) && (byte(c)<=0xAF)) {
        c -= 0x10;
        // Serial.print("DOWN-10");
        ccmd = 1;
      } else {
        if ((byte(c)>=0xD1) && (byte(c)<=0xDF)) {
          c += 0x20;
        } else {
          if( byte(c)==0xD0 ) {
            ccmd = 1;
            if (kbd_lck_state & LED_SCRLLOCK) {
              Serial.print("-OP-");
              c = 0xF0; // keyboard OP mode
            } else {
              Serial.print("-PRO-");
              c= 0xF2; // keyboard PRO mode
            }
          }
        }
      }
/*
      switch (c) {
        case 0x0C: // TAB
          c = 0xF4;
          ccmd = 1;
          break;
        case 0xFF: // Ctrl CMD
          c = 0xFC;
          ccmd = 1;
          break;
        case 0x5A: c = 0x5A;       break;
        default: break;
      }
*/
      if( ccmd == 1 )
        Serial.print("CMD");
      Serial.print("=");
      Serial.print(byte(c));
      Serial.print("=");
      Serial.print(c);
      Serial.println("=");
      sendKey(c, ccmd);
      digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
      delay(2);               // wait for a second
      digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
      delay(10);               // wait for a second
    }
  }
}

